@@ -1,21 +1,24 @@
 
 using System;
+using System.IO;
+using System.Reflection;
 using System.Runtime.InteropServices;
-using Mono.Posix;
+using Mono.Unix;
 
-using PanelApplet;
+// Work around bug in Gtk# panel applet bindings by using a local copy with
+// fixed OnBackgroundChanged marshalling.
+using _Gnome;
 
 namespace Tomboy
 {
-	public class TomboyApplet : PanelApplet.PanelApplet
+	public class TomboyApplet : PanelApplet
 	{
-		static TomboyApplet instance;
-		
 		NoteManager manager;
 		TomboyTray tray;
 		TomboyGConfXKeybinder keybinder;
 
-		BonoboUIVerb [] menu_verbs;
+		// Keep referenced so our callbacks don't get reaped.
+		static BonoboUIVerb [] menu_verbs;
 
 		public TomboyApplet (IntPtr raw)
 			: base (raw)
@@ -34,36 +37,46 @@ namespace Tomboy
 
 		public override void Creation ()
 		{
-			Console.WriteLine ("Applet Created...");
+			Logger.Log ("Applet Created...");
 
 			manager = Tomboy.DefaultNoteManager;
 			tray = new TomboyTray (manager);
 			keybinder = new TomboyGConfXKeybinder (manager, tray);
 
+			Flags |= PanelAppletFlags.ExpandMinor;
+
 			Add (tray);
+			OnChangeSize (Size);
 			ShowAll ();
 
-			// Keep around so our callbacks don't get reaped.
-			menu_verbs = new BonoboUIVerb [] {
-				new BonoboUIVerb (
-					"Props", new ContextMenuItemCallback (ShowPreferencesVerb)),
-				new BonoboUIVerb (
-					"Plugins", new ContextMenuItemCallback (ShowPluginsVerb)),
-				new BonoboUIVerb (
-					"About", new ContextMenuItemCallback (ShowAboutVerb))
-			};
+			if (menu_verbs == null) {
+				menu_verbs = new BonoboUIVerb [] {
+					new BonoboUIVerb ("Plugins", ShowPluginsVerb),
+					new BonoboUIVerb ("Props", ShowPreferencesVerb),
+					new BonoboUIVerb ("Help", ShowHelpVerb),
+					new BonoboUIVerb ("About", ShowAboutVerb)
+				};
+			}
 
-			// FIXME: This silently fails for some unknown reason
-			//SetupMenuFromResource (null, "GNOME_TomboyApplet.xml", menu_verbs);
+			SetupMenuFromResource (null, "GNOME_TomboyApplet.xml", menu_verbs);
+		}
 
-			// Have to resort to this for now
-			SetupMenuFromFile (Defines.DATADIR,
-					   "GNOME_TomboyApplet.xml",
-					   "Tomboy",
-					   menu_verbs);
+		new void SetupMenuFromResource (Assembly asm, 
+						string resource, 
+						BonoboUIVerb [] verbs)
+		{
+			if (asm == null)
+				asm = GetType ().Assembly;
 
-			// Store a reference to avoid segfaults
-			instance = this;
+			Stream stream = asm.GetManifestResourceStream (resource);
+			if (stream != null) {
+				StreamReader reader = new StreamReader (stream);
+				String xml = reader.ReadToEnd ();
+				reader.Close ();
+				stream.Close ();
+
+				SetupMenu (xml, verbs);
+			}
 		}
 
 		void ShowPreferencesVerb ()
@@ -76,41 +89,48 @@ namespace Tomboy
 			manager.PluginManager.ShowPluginsDirectory ();
 		}
 
+		void ShowHelpVerb ()
+		{
+			GuiUtils.ShowHelp("tomboy.xml", null, Screen, null);
+		}
+
 		void ShowAboutVerb ()
 		{
 			tray.ShowAbout ();
 		}
 
-		// NOTE: This is needed to support transparent/pixmap panel
-		//       backgrounds correctly, since Gtk# doesn't map
-		//       GtkStyle::bg_pixmaps[].
-		[DllImport ("libtomboy")]
-		private static extern IntPtr tomboy_widget_set_bg_pixmap (IntPtr widget, 
-									  IntPtr pixmap);
-
-		// FIXME: Overriding to this crashes in the C# bindings.
-		//        Manually tweaked generated bindings to not crash.
-		protected override void OnChangeBackground (BackgroundType type, 
-							    Gdk.Color      color, 
-							    Gdk.Pixmap     pixmap)
+		protected override void OnChangeBackground (PanelAppletBackgroundType type, 
+							    Gdk.Color                 color, 
+							    Gdk.Pixmap                pixmap)
 		{
-			if (tray != null) {
-				Gtk.RcStyle rc_style = new Gtk.RcStyle ();
-				tray.Style = null;
-				tray.ModifyStyle (rc_style);
+			if (tray == null)
+				return;
 
-				switch (type) {
-				case BackgroundType.ColorBackground:
-					tray.ModifyBg (Gtk.StateType.Normal, color);
-					break;
-				case BackgroundType.NoBackground:
-					break;
-				case BackgroundType.PixmapBackground:
-					tomboy_widget_set_bg_pixmap (tray.Handle, pixmap.Handle);
-					break;
-				}
+			Gtk.RcStyle rc_style = new Gtk.RcStyle ();
+			tray.Style = null;
+			tray.ModifyStyle (rc_style);
+
+			switch (type) {
+			case PanelAppletBackgroundType.ColorBackground:
+				tray.ModifyBg (Gtk.StateType.Normal, color);
+				break;
+			case PanelAppletBackgroundType.NoBackground:
+				break;
+			case PanelAppletBackgroundType.PixmapBackground:
+				Gtk.Style copy = tray.Style.Copy();
+				copy.SetBgPixmap (Gtk.StateType.Normal, pixmap);
+				tray.Style = copy;
+				break;
 			}
 		}
+
+		protected override void OnChangeSize (uint size)
+		{
+			if (tray == null)
+				return;
+
+			tray.SetSizeRequest ((int) size, (int) size);
+		}
 	}
 
 	public class TomboyTrayIcon : Gtk.Plug
@@ -155,25 +175,32 @@ namespace Tomboy
 		Gtk.Menu MakeRightClickMenu (Gtk.Widget parent)
 		{
 			Gtk.Menu menu = new Gtk.Menu ();
-			menu.AttachToWidget (parent, null);
+			menu.AttachToWidget (parent, GuiUtils.DetachMenu);
 
 			Gtk.AccelGroup accel_group = new Gtk.AccelGroup ();
 			menu.AccelGroup = accel_group;
 
 			Gtk.ImageMenuItem item;
 
-			item = new Gtk.ImageMenuItem (Catalog.GetString ("_Preferences..."));
-			item.Image = new Gtk.Image (Gtk.Stock.Properties, Gtk.IconSize.Menu);
-			item.Activated += ShowPreferences;
-			menu.Append (item);
-
 			item = new Gtk.ImageMenuItem (Catalog.GetString ("_Open Plugins Folder"));
 			item.Image = new Gtk.Image (Gtk.Stock.Execute, Gtk.IconSize.Menu);
 			item.Activated += ShowPlugins;
 			menu.Append (item);
 
+			menu.Append (new Gtk.SeparatorMenuItem ());
+
+			item = new Gtk.ImageMenuItem (Catalog.GetString ("_Preferences"));
+			item.Image = new Gtk.Image (Gtk.Stock.Preferences, Gtk.IconSize.Menu);
+			item.Activated += ShowPreferences;
+			menu.Append (item);
+
+			item = new Gtk.ImageMenuItem (Catalog.GetString ("_Help"));
+			item.Image = new Gtk.Image (Gtk.Stock.Help, Gtk.IconSize.Menu);
+			item.Activated += ShowHelpContents;
+			menu.Append (item);
+
 			item = new Gtk.ImageMenuItem (Catalog.GetString ("_About Tomboy"));
-			item.Image = new Gtk.Image (Gnome.Stock.About, Gtk.IconSize.Menu);
+			item.Image = new Gtk.Image (Gtk.Stock.About, Gtk.IconSize.Menu);
 			item.Activated += ShowAbout;
 			menu.Append (item);
 
@@ -198,6 +225,11 @@ namespace Tomboy
 			manager.PluginManager.ShowPluginsDirectory ();
 		}
 
+		void ShowHelpContents (object sender, EventArgs args)
+		{
+			GuiUtils.ShowHelp("tomboy.xml", null, Screen, null);
+		}
+
 		void ShowAbout (object sender, EventArgs args)
 		{
 			tray.ShowAbout ();
@@ -205,7 +237,7 @@ namespace Tomboy
 
 		void Quit (object sender, EventArgs args)
 		{
-			Console.WriteLine ("Quitting Tomboy.  Ciao!");
+			Logger.Log ("Quitting Tomboy.  Ciao!");
 			Tomboy.Exit (0);
 		}
 	}
