@@ -1,6 +1,6 @@
 
 using System;
-using Mono.Posix;
+using Mono.Unix;
 using GConf.PropertyEditors;
 
 namespace Tomboy
@@ -13,6 +13,8 @@ namespace Tomboy
 		public const string ENABLE_KEYBINDINGS = "/apps/tomboy/enable_keybindings";
 
 		public const string CUSTOM_FONT_FACE = "/apps/tomboy/custom_font_face";
+		public const string MENU_NOTE_COUNT = "/apps/tomboy/menu_note_count";
+		public const string MENU_PINNED_NOTES = "/apps/tomboy/menu_pinned_notes";
 
 		public const string KEYBINDING_SHOW_NOTE_MENU = "/apps/tomboy/global_keybindings/show_note_menu";
 		public const string KEYBINDING_OPEN_START_HERE = "/apps/tomboy/global_keybindings/open_start_here";
@@ -23,6 +25,8 @@ namespace Tomboy
 		public const string EXPORTHTML_LAST_DIRECTORY = "/apps/tomboy/export_html/last_directory";
 		public const string EXPORTHTML_EXPORT_LINKED = "/apps/tomboy/export_html/export_linked";
 
+		public const string STICKYNOTEIMPORTER_FIRST_RUN = "/apps/tomboy/sticky_note_importer/sticky_importer_first_run";
+
 		static GConf.Client client;
 		static GConf.NotifyEventHandler changed_handler;
 
@@ -54,6 +58,12 @@ namespace Tomboy
 			case CUSTOM_FONT_FACE:
 				return "Serif 11";
 
+			case MENU_NOTE_COUNT:
+				return 10;
+
+			case MENU_PINNED_NOTES:
+				return "";
+
 			case KEYBINDING_SHOW_NOTE_MENU:
 				return "<Alt>F12";
 				
@@ -70,6 +80,9 @@ namespace Tomboy
 
 			case EXPORTHTML_LAST_DIRECTORY:
 				return "";
+
+			case STICKYNOTEIMPORTER_FIRST_RUN:
+				return true;
 			}
 
 			return null;
@@ -110,17 +123,10 @@ namespace Tomboy
 		Gtk.Label font_face;
 		Gtk.Label font_size;
 
-		static Gdk.Pixbuf tintin;
-
-		static PreferencesDialog ()
-		{
-			tintin = new Gdk.Pixbuf (null, "tintin.png");
-		}
-
 		public PreferencesDialog ()
 			: base ()
 		{
-			Icon = tintin;
+			IconName = "tomboy";
 			HasSeparator = false;
 			BorderWidth = 5;
 			Resizable = false;
@@ -179,20 +185,25 @@ namespace Tomboy
 			options_list.Show ();
 
 
-			// Spellchecking...
+			// Spell checking...
 
-			check = MakeCheckButton (Catalog.GetString ("_Spellcheck While Typing"));
-			options_list.PackStart (check, false, false, 0);
+			if (NoteSpellChecker.GtkSpellAvailable) {
+				check = MakeCheckButton (
+					Catalog.GetString ("_Spell check while typing"));
+				options_list.PackStart (check, false, false, 0);
+				
+				peditor = new PropertyEditorToggleButton (
+					Preferences.ENABLE_SPELLCHECKING,
+					check);
+				SetupPropertyEditor (peditor);
 
-			peditor = new PropertyEditorToggleButton (Preferences.ENABLE_SPELLCHECKING,
-								  check);
-			SetupPropertyEditor (peditor);
-
-			label = MakeTipLabel (Catalog.GetString ("Misspellings will be underlined " +
-								 "in red, and correct spelling " +
-								 "suggestions shown in the right-click " +
-								 "menu."));
-			options_list.PackStart (label, false, false, 0);
+				label = MakeTipLabel (
+					Catalog.GetString ("Misspellings will be underlined " +
+							   "in red, and correct spelling " +
+							   "suggestions shown in the context " +
+							   "menu."));
+				options_list.PackStart (label, false, false, 0);
+			}
 
 
 			// WikiWords...
@@ -213,7 +224,7 @@ namespace Tomboy
 
 			// Custom font...
 
-			check = MakeCheckButton (Catalog.GetString ("Use Custom _Font"));
+			check = MakeCheckButton (Catalog.GetString ("Use custom _font"));
 			options_list.PackStart (check, false, false, 0);
 
 			font_peditor = 
@@ -223,7 +234,7 @@ namespace Tomboy
 
 			align = new Gtk.Alignment (0.5f, 0.5f, 0.4f, 1.0f);
 			align.Show ();
-			options_list.PackStart (align, true, true, 0);
+			options_list.PackStart (align, false, false, 0);
 
 			font_button = MakeFontButton ();
 			font_button.Sensitive = check.Active;
@@ -315,6 +326,7 @@ namespace Tomboy
 			table.Attach (label, 0, 1, 0, 1);
 
 			entry = new Gtk.Entry ();
+			label.MnemonicWidget = entry;	
 			entry.Show ();
 			table.Attach (entry, 1, 2, 0, 1);
 
@@ -331,6 +343,7 @@ namespace Tomboy
 			table.Attach (label, 0, 1, 1, 2);
 
 			entry = new Gtk.Entry ();
+			label.MnemonicWidget = entry;	
 			entry.Show ();
 			table.Attach (entry, 1, 2, 1, 2);
 
@@ -347,6 +360,7 @@ namespace Tomboy
 			table.Attach (label, 0, 1, 2, 3);
 
 			entry = new Gtk.Entry ();
+			label.MnemonicWidget = entry;	
 			entry.Show ();
 			table.Attach (entry, 1, 2, 2, 3);
 
@@ -357,16 +371,17 @@ namespace Tomboy
 			keybind_peditor.AddGuard (entry);
 
 
-			// Search dialog keybinding...
+			// Open All Notes window keybinding...
 
-			label = MakeLabel (Catalog.GetString ("S_earch notes"));
+			label = MakeLabel (Catalog.GetString ("Open \"_Table of Contents\""));
 			table.Attach (label, 0, 1, 3, 4);
 
 			entry = new Gtk.Entry ();
+			label.MnemonicWidget = entry;	
 			entry.Show ();
 			table.Attach (entry, 1, 2, 3, 4);
 
-			peditor = new PropertyEditorEntry (Preferences.KEYBINDING_OPEN_SEARCH, 
+			peditor = new PropertyEditorEntry (Preferences.KEYBINDING_OPEN_RECENT_CHANGES, 
 							   entry);
 			SetupPropertyEditor (peditor);
 
@@ -409,7 +424,7 @@ namespace Tomboy
 
 		Gtk.Label MakeTipLabel (string label_text)
 		{
-			Gtk.Label label =  MakeLabel (String.Format ("<small><i>{0}</i></small>", 
+			Gtk.Label label =  MakeLabel (String.Format ("<small>{0}</small>", 
 								     label_text));
 			label.LineWrap = true;
 			label.Xpad = 20;
