@@ -1,24 +1,19 @@
 
 using System;
 using System.IO;
-using Mono.Posix;
+using Mono.Unix;
 
 namespace Tomboy 
 {
-	public class Tomboy 
+	public class Tomboy : Application
 	{
-		static Gnome.Program program;
-		static Syscall.sighandler_t sig_handler;
 		static NoteManager manager;
-		static Object dbus_connection;
+#if ENABLE_DBUS
+		static RemoteControl remote_control;
+#endif
 
 		public static void Main (string [] args) 
 		{
-			RegisterSignalHandlers ();
-
-			// Initialize GETTEXT
-			Catalog.Init ("tomboy", Defines.GNOME_LOCALE_DIR);
-
 			TomboyCommandLine cmd_line = new TomboyCommandLine (args);
 
 			if (cmd_line.NeedsExecute) {
@@ -27,28 +22,47 @@ namespace Tomboy
 				return;
 			}
 
-			program = new Gnome.Program ("Tomboy", 
-						     Defines.VERSION, 
-						     Gnome.Modules.UI, 
-						     args);
+			Initialize ("tomboy", "Tomboy", "tomboy", args);
 
 			// Create the default note manager instance.
-			if (cmd_line.NotePath != null) {
-				manager = new NoteManager (cmd_line.NotePath);
-			} else {
-				manager = new NoteManager ();
-			}
+			string note_path = GetNotePath (cmd_line.NotePath);
+			manager = new NoteManager (note_path);
 
 			// Register the manager to handle remote requests.
 			RegisterRemoteControl (manager);
 
 			if (cmd_line.UsePanelApplet) {
-				RegisterPanelAppletFactory (); 
+				RegisterPanelAppletFactory ();
 			} else {
+				RegisterSessionManagerRestart (
+					Environment.GetEnvironmentVariable ("TOMBOY_WRAPPER_PATH"),
+					args,
+					new string [] { "TOMBOY_PATH=" + note_path  });
 				StartTrayIcon ();
 			}
 
-			Console.WriteLine ("All done.  Ciao!");
+			Logger.Log ("All done.  Ciao!");
+		}
+
+		static string GetNotePath (string override_path)
+		{
+			// Default note location, as specified in --note-path or $TOMBOY_PATH
+			string note_path = 
+				(override_path != null) ? 
+		                       override_path : 
+				       Environment.GetEnvironmentVariable ("TOMBOY_PATH");
+			if (note_path == null)
+				note_path = "~/.tomboy";
+
+			// Tilde expand
+			return note_path.Replace ("~", Environment.GetEnvironmentVariable ("HOME"));
+		}
+
+		static void RegisterPanelAppletFactory ()
+		{
+			// This will block if there is no existing instance running
+			// FIXME: Use custom built panel applet bindings to work around bug in GTK#
+			_Gnome.PanelAppletFactory.Register (typeof (TomboyApplet));
 		}
 
 		static void StartTrayIcon ()
@@ -56,62 +70,28 @@ namespace Tomboy
 			// Create the tray icon and run the main loop
 			TomboyTrayIcon tray_icon = new TomboyTrayIcon (DefaultNoteManager);
 			tray_icon.Show ();
-			program.Run ();
-		}
 
-		static void RegisterPanelAppletFactory ()
-		{
-			// This will block if there is no existing instance running
-			PanelApplet.AppletFactory.Register (typeof (TomboyApplet));
+			StartMainLoop ();
 		}
 
 		static void RegisterRemoteControl (NoteManager manager)
 		{
 #if ENABLE_DBUS
 			try {
-				dbus_connection = DBus.Bus.GetSessionBus ();
-				DBus.Service service = 
-					new DBus.Service ((DBus.Connection) dbus_connection, 
-							  RemoteControlProxy.Namespace);
-
-				RemoteControl remote_control = new RemoteControl (manager);
-				service.RegisterObject (remote_control, RemoteControlProxy.Path);
-
-				Console.WriteLine ("Tomboy remote control active.");
+				remote_control = RemoteControlProxy.Register (manager);
+				if (remote_control != null) {
+					Logger.Log ("Tomboy remote control active.");
+				} else {
+					Logger.Log ("Tomboy is already running.  Exiting...");
+					System.Environment.Exit (-1);
+				}
 			} catch (Exception e) {
-				Console.WriteLine ("Tomboy remote control disabled: {0}",
+				Logger.Log ("Tomboy remote control disabled (DBus exception): {0}",
 						   e.Message);
 			}
 #endif
 		}
 
-		static void RegisterSignalHandlers ()
-		{
-			sig_handler = OnExitSignal;
-
-			// Connect to SIGTERM and SIGQUIT, so we don't lose
-			// unsaved notes on exit...
-			Syscall.signal ((int) Signals.SIGTERM, sig_handler);
-			Syscall.signal ((int) Signals.SIGQUIT, sig_handler);
-		}
-
-		static void OnExitSignal (int signal)
-		{
-			if (ExitingEvent != null)
-				ExitingEvent (null, new EventArgs ());
-
-			if (signal >= 0)
-				System.Environment.Exit (0);
-		}
-
-		public static event EventHandler ExitingEvent;
-
-		public static void Exit (int exitcode)
-		{
-			OnExitSignal (-1);
-			System.Environment.Exit (exitcode);
-		}
-
 		public static NoteManager DefaultNoteManager
 		{
 			get { return manager; }
@@ -127,6 +107,8 @@ namespace Tomboy
 		string open_note_name;
 		string highlight_search;
 		string note_path;
+		string search_text;
+		bool open_search;
 
 		public TomboyCommandLine (string [] args)
 		{
@@ -141,7 +123,10 @@ namespace Tomboy
 		public bool NeedsExecute
 		{
 			get { 
-				return new_note || open_note_name != null || open_note_uri != null;
+				return new_note || 
+						open_note_name != null ||
+						open_note_uri != null || 
+						open_search;
 			}
 		}
 
@@ -182,7 +167,9 @@ namespace Tomboy
 					"matching title.\n" +
 					"  --start-here\t\t\tDisplay the 'Start Here' note.\n" +
 					"  --highlight-search [text]\tSearch and highlight text " +
-					"in the opened note.\n");
+					"in the opened note.\n" +
+					"  --search [text]\tOpen the search all notes window with" +
+					"the search text.\n");
 #else
 			usage += Catalog.GetString ("D-BUS remote control disabled.\n");
 #endif
@@ -243,11 +230,21 @@ namespace Tomboy
 					++idx;
 					highlight_search = args [idx];
 					break;
+
+				case "--search":
+					// Get optional search text...
+					if (idx + 1 < args.Length && args [idx + 1][0] != '-') {
+						search_text = args [++idx];
+					}
+					
+					open_search = true;
+					break;
 #else
 				case "--new-note":
 				case "--open-note":
 				case "--start-here":
 				case "--highlight-search":
+				case "--search":
 					string unknown_opt = 
 						Catalog.GetString (
 							"Tomboy: unsupported option '{0}'\n" +
@@ -306,13 +303,16 @@ namespace Tomboy
 		public void Execute ()
 		{
 #if ENABLE_DBUS
-			DBus.Connection connection = DBus.Bus.GetSessionBus ();
-			DBus.Service service = new DBus.Service (connection, 
-								 RemoteControlProxy.Namespace);
+			RemoteControl remote = null;
+			try {
+				remote = RemoteControlProxy.GetInstance ();
+			} catch (Exception e) {
+				Logger.Log ("Unable to connect to Tomboy remote control: {0}",
+					e.Message);
+			}
 
-			RemoteControlProxy remote = (RemoteControlProxy) 
-				service.GetObject (typeof (RemoteControlProxy),
-						   RemoteControlProxy.Path);
+			if (remote == null)
+				return;
 
 			if (new_note) {
 				string new_uri;
@@ -339,6 +339,13 @@ namespace Tomboy
 				else
 					remote.DisplayNote (open_note_uri);
 			}
+			
+			if (open_search) {
+				if (search_text != null)
+					remote.DisplaySearchWithText (search_text);
+				else
+					remote.DisplaySearch ();
+			}
 #endif // ENABLE_DBUS
 		}
 	}
